from twisted.cred import portal, checkers
from twisted.conch import manhole, manhole_ssh
from twisted.internet import reactor
import logging

def manhole_factory(nmspace, **passwords):
    realm = manhole_ssh.TerminalRealm()

    def getManhole(_):
        return manhole.Manhole(nmspace)

    realm.chainedProtocolFactory.protocolFactory = getManhole
    p = portal.Portal(realm)
    p.registerChecker(
        checkers.InMemoryUsernamePasswordDatabaseDontUse(**passwords)
    )
    f = manhole_ssh.ConchFactory(p)
    return f

# how to log in: ssh user_name@localhost -p port
def start_manhole(droid, nmspace, cliargs):
    """
    Start EnDroid listening for an ssh connection.

    Logging in via ssh gives access to a python prompt from which EnDroid's
    internals can be investigated.

    Login details are specified in the cliargs tuple, which contains:
    (login_name, login_password, port)

    """
    logging.info("Starting manhole")
    # room to expand / add functionality to the manhole_dict here
    manhole_dict = dict(locals().items() + globals().items())
    login, password, port = cliargs
    reactor.listenTCP(int(port), manhole_factory(nmspace, **{login: password}))
    droid.run()
 
