# -----------------------------------------------------------------------------
# EnDroid - Help Plugin
# Copyright 2012, Ensoft Ltd
# -----------------------------------------------------------------------------

from endroid.plugins.command import CommandPlugin

class Help(CommandPlugin):
    def endroid_init(self):
        self.help_topics = {
            '': self.show_help_main,
            'plugins': self.show_help_plugins,
            'commands': self.show_help_commands,
            'help': lambda _: "DON'T PANIC!",
            '*': lambda t: self.show_help_plugin(*t.strip().split(' ', 1)),
            }

        self.load_plugin_list()

    def load_plugin_list(self):
        self.plugins = {}
        for fullname in self.list_plugins():
            plugin = self.get(fullname)
            name = getattr(plugin, "name", fullname)
            self.plugins[name] = (fullname, plugin)

    def cmd_help(self, msg, args):
        msg.reply_to_sender(self.show_help_plugin("help", args))

    def show_help_main(self, topic):
        assert not topic
        out = []
        out.append("EnDroid at your service! Help topics:")
        out.append("  plugins - list loaded plugins")
        out.append("  commands - list supported commands")
        out.append("  <pluginname> - help provided by the given plugin")
        return "\n".join(out)

    def show_help_plugins(self, topic):
        if topic.strip():
            # specific plugin?
            return self.help_topics['*'](topic)
        else:
            out = []
            out.append("Currently loaded plugins:")
            for name, (_, plug) in sorted(self.plugins.items()):
                if not getattr(plug, "hidden", False):
                    out.append("  {0}".format(name))
            return "\n".join(out)

    def show_help_commands(self, topic):
        return self.show_help_plugin("commands", topic)

    def show_help_plugin(self, name, topic=''):
        out = []
        fullname, plugin = self.plugins.get(name, (name, None))
        if self.pluginLoaded(fullname):
            # First check if it has a simple "help" property or method
            if plugin.hasattr("help"):
                try:
                    out.append(plugin.help(topic))
                except TypeError:
                    if topic:
                        out.append("(Plugin doesn't support topic help)")
                    if callable(plugin.help):
                        out.append(plugin.help())
                    else:
                        out.append(str(plugin.help))

            elif plugin.hasattr("help_topics"):
                # Check if it is a "help_topics" dictionary, mapping topic
                # (first keyword) to handler function
                keywords = topic.strip().split(' ', 1)
                topic, extra = keywords[0], ''.join(keywords[1:])

                if topic in plugin.help_topics:
                    out.append(plugin.help_topics[topic](extra))
                elif '*' in plugin.help_topics:
                    out.append(plugin.help_topics['*'](' '.join(keywords)))
                else:
                    out.append("Unknown topic '{0}' for plugin {1}".format(topic,
                                                                           name))
            else:
                out.append("Plugin '{0}' provides no help".format(name))
        else:
            out.append("Plugin '{0}' is not loaded".format(name))

        return '\n'.join(out)
